' -------------------------------------------------------------------------
'       From the book Inside Active Directory, ISBN 0-201-61621-1
'       Copyright (C) 2002 by Addison-Wesley
'       Script by Sakari Kouti (see http://www.kouti.com)
' You have a royalty-free right to use, modify, reproduce and distribute
' this script (and/or any modified version) in any way you find useful,
' provided that you agree that Addison-Wesley or Sakari Kouti has no
' warranty, obligations or liability for the script. If you modify
' the script, you must retain this copyright notice.
' -------------------------------------------------------------------------
Option Explicit

'AccessMask
Const ADS_RIGHT_DS_CREATE_CHILD        = &H1
Const ADS_RIGHT_DS_DELETE_CHILD        = &H2
Const ADS_RIGHT_ACTRL_DS_LIST          = &H4
Const ADS_RIGHT_DS_SELF                = &H8
Const ADS_RIGHT_DS_READ_PROP           = &H10
Const ADS_RIGHT_DS_WRITE_PROP          = &H20
Const ADS_RIGHT_DS_DELETE_TREE         = &H40
Const ADS_RIGHT_DS_LIST_OBJECT         = &H80
Const ADS_RIGHT_DS_CONTROL_ACCESS      = &H100
Const ADS_RIGHT_DELETE                 = &H10000
Const ADS_RIGHT_READ_CONTROL           = &H20000
Const ADS_RIGHT_WRITE_DAC              = &H40000
Const ADS_RIGHT_WRITE_OWNER            = &H80000
Const ADS_RIGHT_SYNCHRONIZE            = &H100000
Const ADS_RIGHT_ACCESS_SYSTEM_SECURITY = &H1000000
Const ADS_RIGHT_GENERIC_ALL            = &H10000000
Const ADS_RIGHT_GENERIC_EXECUTE        = &H20000000
Const ADS_RIGHT_GENERIC_WRITE          = &H40000000
Const ADS_RIGHT_GENERIC_READ           = &H80000000

Const ADS_RIGHT_FULL_CONTROL = &HF01FF

'ACE flags
Const ADS_ACEFLAG_INHERIT_ACE = &H2
Const ADS_ACEFLAG_NO_PROPAGATE_INHERIT_ACE = &H4
Const ADS_ACEFLAG_INHERIT_ONLY_ACE = &H8
Const ADS_ACEFLAG_INHERITED_ACE = &H10
Const ADS_ACEFLAG_SUCCESSFUL_ACCESS = &H40
Const ADS_ACEFLAG_FAILED_ACCESS = &H80

'ACE types
Const ADS_ACETYPE_ACCESS_ALLOWED           = 0
Const ADS_ACETYPE_ACCESS_DENIED            = &H1
Const ADS_ACETYPE_SYSTEM_AUDIT             = &H2
Const ADS_ACETYPE_ACCESS_ALLOWED_OBJECT    = &H5
Const ADS_ACETYPE_ACCESS_DENIED_OBJECT     = &H6
Const ADS_ACETYPE_SYSTEM_AUDIT_OBJECT      = &H7

'Flags
Const ADS_FLAG_OBJECT_TYPE_PRESENT           = &H1
Const ADS_FLAG_INHERITED_OBJECT_TYPE_PRESENT = &H2

'Some schemaIDGUIDs
Const SCHEMAIDGUID_USER = _
	"{BF967ABA-0DE6-11D0-A285-00AA003049E2}"
Const SCHEMAIDGUID_GROUP = _
	"{BF967A9C-0DE6-11D0-A285-00AA003049E2}"
Const SCHEMAIDGUID_COMPUTER = _
	"{BF967A86-0DE6-11D0-A285-00AA003049E2}"
Const SCHEMAIDGUID_CONTACT = _
	" {5CB41ED0-0E4C-11D0-A286-00AA003049E2}"
Const SCHEMAIDGUID_OU = _
	"{BF967AA5-0DE6-11D0-A285-00AA003049E2}"
Const SCHEMAIDGUID_PRINTER = _
	"{BF967AA8-0DE6-11D0-A285-00AA003049E2}"

Dim objDSE, objDom, objOU, objSecDesc, objDACL
Dim objACE1, objACE2, objACE3

'======================

'Create the OU
'---------------------------
Set objDSE = GetObject("LDAP://rootDSE")
Set objDom = GetObject("LDAP://" & _
	objDSE.Get("defaultNamingContext"))
Set objOU = objDom.Create("organizationalUnit", "OU=ACEDemo")
objOU.SetInfo

'Create the first ACE - write all properties of the OU
'------------------------------------------------------------
Set objACE1 = CreateObject("AccessControlEntry")
objACE1.Trustee			= "Red.Baron@sanao.com"
objACE1.AccessMask		= ADS_RIGHT_DS_WRITE_PROP
objACE1.AceFlags		= 0	'no inheritance from up or to below
objACE1.AceType			= ADS_ACETYPE_ACCESS_ALLOWED
objACE1.Flags			= 0	'no object types present
'objACE1.ObjectType		= not used
'objACE1.InheritedObjectType = not used

'Create the second ACE - Full Control to user objects in the subtree
'---------------------------------------------------------------------------
Set objACE2 = CreateObject("AccessControlEntry")
objACE2.Trustee			=	"Red.Baron@sanao.com"
objACE2.AccessMask		=	ADS_RIGHT_FULL_CONTROL  'all 13 bits
objACE2.AceFlags		=	ADS_ACEFLAG_INHERIT_ACE + _
							ADS_ACEFLAG_INHERIT_ONLY_ACE
objACE2.AceType			=	ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
objACE2.Flags			=	ADS_FLAG_INHERITED_OBJECT_TYPE_PRESENT
'objACE2.ObjectType		=	not used
objACE2.InheritedObjectType = SCHEMAIDGUID_USER

'Create the third ACE - Create/Delete user objects in the subtree
'-------------------------------------------------------------------------
Set objACE3 = CreateObject("AccessControlEntry")
objACE3.Trustee			=	"Red.Baron@sanao.com"
objACE3.AccessMask		=	ADS_RIGHT_DS_CREATE_CHILD + _
							ADS_RIGHT_DS_DELETE_CHILD
objACE3.AceFlags		=	ADS_ACEFLAG_INHERIT_ACE
objACE3.AceType			=	ADS_ACETYPE_ACCESS_ALLOWED_OBJECT
objACE3.Flags			=	ADS_FLAG_OBJECT_TYPE_PRESENT
objACE3.ObjectType		=	SCHEMAIDGUID_USER
'objACE3.InheritedObjectType = not used

'Add the ACEs
'---------------------------
Set objSecDesc = objOU.Get("ntSecurityDescriptor")
Set objDACL = objSecDesc.DiscretionaryAcl

Call objDACL.AddAce(objACE1)
Call objDACL.AddAce(objACE2)
Call objDACL.AddAce(objACE3)

objSecDesc.DiscretionaryAcl = objDACL
Call objOU.Put("ntSecurityDescriptor", objSecDesc)
objOU.SetInfo
	
WScript.Echo "Number of ACEs after: " & objDACL.AceCount
