' -------------------------------------------------------------------------
'       From the book Inside Active Directory, ISBN 0-201-61621-1
'       Copyright (C) 2002 by Addison-Wesley
'       Script by Sakari Kouti (see http://www.kouti.com)
' You have a royalty-free right to use, modify, reproduce and distribute
' this script (and/or any modified version) in any way you find useful,
' provided that you agree that Addison-Wesley or Sakari Kouti has no
' warranty, obligations or liability for the script. If you modify
' the script, you must retain this copyright notice.
' -------------------------------------------------------------------------
Option Explicit

Const ERROR_OBJECT_ALREADY_EXISTS = &H80071392
Const ERROR_DS_NO_SUCH_OBJECT = &H80072030

Dim strName, objContainer, objNewUser

On Error Resume Next

strName = "Jack"

Err.Clear
Set objContainer = _
    GetObject("LDAP://OU=Sales,DC=sanao,DC=com")
Call CheckIfError("Bind to OU=Sales")

Err.Clear
Set objNewUser = objContainer.Create("user", "CN=" & strName)
objNewUser.sAMAccountName = strName
objNewUser.userAccountControl = &H200
objNewUser.userPrincipalName = strName & "@sanao.com"
objNewUser.SetInfo
Call CheckIfError("Create user " & strName)

Err.Clear
WScript.Echo "Last name " & objNewUser.Get("sn")
Call CheckIfError("Read the last name of " & strName)

Err.Clear
objNewUser.SetPassword ("secret")
Call CheckIfError("Set password for user " & strName)

Sub CheckIfError(strOperation)
    Dim strMessage, intErrCode

    If Err = 0 Then Exit Sub

    strMessage = "Attempted operation: " & strOperation & vbCrLf

    Select Case Err
        Case ERROR_OBJECT_ALREADY_EXISTS
            strMessage = strMessage & _
                "Unfortunately the object to be created already exists."
        Case ERROR_DS_NO_SUCH_OBJECT
            strMessage = strMessage & _
                "Unfortunately couldn't find that object."
        Case Else
            strMessage = strMessage & _
                "Error code: " & Hex(Err) & " (hex)" & vbCrLf & _
                Err.Description
            If (Err And &HFFFF0000) = &H80070000 Then
                intErrCode = (Err And 65535)  '65535 = &HFFFF
                strMessage = strMessage & vbCrLf & _
                    "You can get more help on the error by typing" & vbCrLf & _
                    "on the command line: NET HELPMSG " & intErrCode
            End If
    End Select

    MsgBox strMessage & vbCrLf & vbCrLf & _
        "Click OK to terminate the script.", _
        vbOKOnly + vbCritical, "Error"

    WScript.Quit(1)	'there was an error so we set the errorlevel
End Sub
